(ns poalses.jonathan.dialect.dialect-nlp
  (:require [clojure.datafy :refer [datafy]]
           [dk.simongray.datalinguist :as dl]
           [dk.simongray.datalinguist.triple :refer [triple->datalog]])
  (:import [edu.stanford.nlp.coref CorefCoreAnnotations$CorefChainAnnotation]))


(def example
  "Great work, thanks for doing this!
  Having had no previous experience with NLP libraries, I was wondering why I couldn’t get your examples to work. Then I realized that I had to download CoreNLP first from https://stanfordnlp.github.io/CoreNLP/ and add stanford-corenlp-4.4.0/* to the classpath. Everything worked fine after that.
  Is this what you are supposed to do? It wasn’t mentioned in the readme, so I was wondering if I did something wrong here or if it is more obvious to people who have already worked with CoreNLP.")

(def test-sentence-one
  "This is an example of correct language usage. I had an idea about using garlic butter with chicken, hence the food was burnt.")

(def test-sentence-two
  "This is an example of incorrect language usage. I had an idea about using garlic butter with chicken, hence why the food was burnt.")

(def nlp
  (dl/->pipeline {:annotators ["truecase"
                               "quote"
                               "entitymentions"
                               "parse"
                               "depparse"
                               "lemma"
                               "relation"
                               "tokenize"
                               "coref"
                               "openie"
                               "ner"]
                  :quote      {:extractUnclosedQuotes "true"}}))

;; Take a dependency graph object and change it into a map of tokens and parts of speech labels

(defn dependency-to-token-pos-map [graph])


;; Take a dependency graph object and change it into a map of tokens and relation labels

(defn dependency-to-token-relation-map [graph])


;; Take a text sample and separate it into it's sentences, then for each sentence find its dialect, and return the most common dialect

(defn detect-sample-dialect [sample])


;; Take a sentence and figure out its dialect

(defn detect-sentence-dialect [sentence])



(def annotated-example
  (delay (nlp example)))

(def sentences
  (delay (dl/sentences @annotated-example)))

(def annotated-example-one
  (delay (nlp test-sentence-one)))

(def sentences-one
  (dl/sentences @annotated-example-one))

(def annotated-example-two
  (delay (nlp test-sentence-two)))

(def sentences-two
  (delay (dl/sentences @annotated-example-two)))

(defn show-dependencies []
  (clojure.pprint/pprint (dl/dependency-graph (nth sentences-one 1))))


(comment
  ;; Test every annotator in the pipeline
  (map dl/true-case @sentences)
  (map dl/quotations @sentences)
  (map dl/mentions @sentences)
  (map dl/annotation "relation" @sentences)
  (map dl/constituency-tree @sentences)
  (map dl/constituency-tree sentences-one)
  (map dl/constituency-tree @sentences-two)
  (map dl/dependency-graph @sentences)
  (dl/dependency-graph (nth sentences-one 1))
  (map dl/dependency-graph @sentences-two)
  (map dl/lemma @sentences)
  (map dl/lemma sentences-one)
  (map dl/lemma @sentences-two)
  (dl/text (dl/tokens (nth sentences-one 1)))
  (map dl/tokens @sentences-two)
  (->> (mapcat dl/triples @sentences) (map triple->datalog))
  (dl/annotation CorefCoreAnnotations$CorefChainAnnotation @annotated-example)

  (show-dependencies)

  (dl/text sentences-one)
  (dl/triples sentences-one)
  (dl/triples (dl/dependency-graph (nth sentences-one 1)))

  ;; Datafy the annotations. Retrieves direct annotations for every sentence.
  ;; Keep in mind that `dl/recur-datafy` currently doesn't work in this instance
  ;; and will possibly be removed in a future update:
  ;;   https://github.com/simongray/datalinguist/issues/13
  (into {} (map datafy @sentences))
  (keys (into {} (map datafy @sentences)))
  (:natural-logic/relation-triples (into {} (map datafy @sentences)))
  (:semantic-graph/enhanced-plus-plus-dependencies (into {} (map datafy @sentences)))
  (:semantic-graph/collapsed-dependencies (into {} (map datafy @sentences)))
  (:coref/mentions (into {} (map datafy @sentences)))
  (:tokens (into {} (map datafy @sentences)))
  (def play-map (into {} (map datafy @sentences)))
  (def play-map (into {} (datafy @annotated-example)))

  sentences
  @sentences

  play-map

  (:text play-map)
  #_.)